/*-----copyright-----
 *     amy-studio - make development easier 
 *     web        : http://www.amy-studio.com
 *     taobao     : http://amy-studio.taobao.com
 *     qq(e_mail) : amystudio@qq.com
 *-----file info-----
 *     name       : key_debounce.v
 *     author     : Yuphone Chang
 *     description: key debounce
 *----- version -----
 *     created    : 7.19, 2010
 *     version    : 1.0
 *     description: original version
 *
 *     modified   : 
 *     version    : 
 *     description: 
 *-------------------
 */

module key_debounce
#(parameter KEY_WIDTH = 4)
(
  input                    i_clk,
  input                    i_rst_n,
  input      [KEY_WIDTH:1] i_key,       // When pressed, input 0; otherwise, input 1
  output reg [KEY_WIDTH:1] o_key_val    // KEYs' value
);  

//++++++++++++++++++++++++++++++++++++++
reg [KEY_WIDTH:1] key_samp1, key_samp1_locked;

// Sample the value of i_key into key_samp1
always @ (posedge i_clk, negedge i_rst_n)
  if(!i_rst_n) 
    key_samp1 <= {KEY_WIDTH{1'b1}};
  else         
    key_samp1 <= i_key;

// Lock the value of key_samp1 into key_samp1_locked
always @ (posedge i_clk, negedge i_rst_n)
  if(!i_rst_n) 
    key_samp1_locked <= {KEY_WIDTH{1'b1}};
  else         
    key_samp1_locked <= key_samp1;
//--------------------------------------

//++++++++++++++++++++++++++++++++++++++
wire [KEY_WIDTH:1] key_changed1;      // Key changing flag 1

// Falling edge detection
assign key_changed1 = key_samp1_locked & (~key_samp1); 
//--------------------------------------


//++++++++++++++++++++++++++++++++++++++
reg [19:0] cnt;

// Once any key is pressed, cnt will be clean.
// So only when the key is stable(without bounce), 
// the counter works normally.
always @ (posedge i_clk, negedge i_rst_n)
  if(!i_rst_n)
    cnt <= 20'h0;
  else if(key_changed1)
    cnt <= 20'h0;
  else
    cnt <= cnt + 1'b1;
//--------------------------------------


//++++++++++++++++++++++++++++++++++++++
reg [KEY_WIDTH:1] key_samp2, key_samp2_locked;

// Only when the key is stable(without bounce), 
// and maintains for beyond 20ms,
// sample the value of i_key into key_samp2.
always @ (posedge i_clk, negedge i_rst_n)
  if(!i_rst_n)
    key_samp2 <= {KEY_WIDTH{1'b1}};
  else if(cnt == 20'hF_FFFF)            // 0xFFFFF/50M = 20.9715ms
    key_samp2 <= i_key;

// Lock the value of key_samp2 into key_samp2_locked.
always @ (posedge i_clk, negedge i_rst_n)
  if(!i_rst_n)
    key_samp2_locked <= {KEY_WIDTH{1'b1}};
  else
    key_samp2_locked <= key_samp2;
//--------------------------------------

//++++++++++++++++++++++++++++++++++++++
wire [KEY_WIDTH:1] key_changed2;        // Key changing flag 2

// Falling edge detection
assign key_changed2 = key_samp2_locked & (~key_samp2); 
//--------------------------------------


//++++++++++++++++++++++++++++++++++++++
// When KEYs is stable, output their value. 
// Pressed value is 0, and released value is 1.
always @ (posedge i_clk, negedge i_rst_n)
  if(!i_rst_n)
    o_key_val <= {KEY_WIDTH{1'b1}};
  else
    o_key_val <= ~key_changed2;
//--------------------------------------

endmodule 
